/*
 * Created on 20-feb-2006
 */
package elab_stats;

import sudoku_human.Utils;



/**
 * Classe per raccogliere dati statistici sull'esecuzione degli algoritmi di risoluzione
 * del sudoku, processarli e stampare i risultati delle elaborazioni.
 * Mantiene sempre l'id del problema corrente, per il quale raccoglie i dati
 * @author Marco Cornolti
 *
 */
public class Statistiche {
    private static int arrLength=1000;
    private static int[] tempi = new int[arrLength];
    private static int[] espansioni = new int[arrLength];
    private static int currEl = 0;
    private static int risolti = 0;
    
    /**
     * aumento del numero di espansioni di 1
     */
    public static void aumentaEspansioni(){
        espansioni[currEl]++;
    }
    /** aumento delle espansioni di un certo valore
     * @param n quantita' di espansioni svolte.
     */
    public static void aumentaEspansioni(int n){
        espansioni[currEl]+=n;
    }
    /**
     * aumento dei problemi risolti.
     */
    public static void aumentaRisolti(){
        risolti++;
    }
    /**assegnamento del tempo trascorso per la risoluzione del problema corrente.
     * @param msec
     */
    public static void setTempo(int msec){
       tempi[currEl]=msec;
    }
    
    /**
     * passa alla raccolta dei dati per il problema successivo.
     */
    public static void nextEl(){
        if (currEl+1==arrLength){
            arrLength*=2;
            int[] newTempi = new int[arrLength];
            int[] newEsp= new int[arrLength];
            for (int i=0;i<arrLength/2;i++){
                newTempi[i]=tempi[i];
                newEsp[i]=espansioni[i];
            }
            espansioni=newEsp;
            tempi=newTempi;
        }
        currEl++;
    }
    
    /**
     * @return la somma dei tempi necessari a risolvere tutti i problemi.
     */
    private static long tempoGlobale(){
        long result=0;
        for (int i=0;i<currEl;i++) result+=tempi[i];
        return result;
    }
    /**
     * @return la somma delle espansioni necessarie a risolvere tutti i problemi.
     */
    private static long espGlobale(){
        long result=0;
        for (int i=0;i<currEl;i++) result+=espansioni[i];
        return result;
    }
    
    
    /**
     * stampa le statistiche raccolte.
     */
    public static void printSummary(){
        long tempoG = tempoGlobale();
        long espG = espGlobale();
        double mediaT = (double)tempoG / (double)currEl;
        double mediaE = (double)espG / (double)currEl;
        double varianzaT=0;
        int maxTempoI=0;
        int maxEspI=0;
        
        
        for (int i=0;i<currEl;i++) {
            varianzaT += Math.pow(mediaT-(double)tempi[i],2);
            if (tempi[maxTempoI]<=tempi[i]){
                maxTempoI=i;
            }
            if (espansioni[maxEspI]<=espansioni[i]){
                maxEspI=i;
            }
        }
        varianzaT /= (double)currEl;
        
        System.out.println(" --- Statistiche sulle soluzioni ---");
        System.out.println("Problemi iniziati: "+currEl);
        System.out.println("Problemi risolti: "+risolti);
        System.out.println("Problemi non risolti: "+(currEl-risolti));
        
        System.out.println("\n --- Statistiche sui tempi ---");
        System.out.println("Tempo globale impiegato: " + tempoG);
        System.out.println("Media tempo impiegato: " + mediaT);
        System.out.println("Varianza tempo impiegato: " + varianzaT);
        System.out.println("Massimo tempo impiegato: " + tempi[maxTempoI] +" nell'istanza n."+ maxTempoI);
        System.out.println("Tempo impiegato per risolvere l'istanza col massimo di espansioni ("+ maxEspI +"): " + tempi[maxEspI]);
        System.out.println("\n --- Statistiche sulle espansioni ---");
        System.out.println("Globale espansioni: " + espG);
        System.out.println("Media espansioni: " + mediaE);
        System.out.println("Massimo numero di espansioni in un'istanza: " + espansioni[maxEspI] +" nell'istanza n."+ maxEspI);
        System.out.println("Numero di espansioni nell'istanza col tempo massimo ("+ maxTempoI +"): " + espansioni[maxTempoI]);
        
        
        System.out.println("\n --- Classifiche sui tempi - primi 10 ---");
        int[] posizioni = new int[arrLength];
        for (int i=0; i<posizioni.length;i++) posizioni[i]=i;
        Utils.sortByWeightDecrease(tempi, posizioni);
        
        for (int i=0;tempi[i]!=0 && i<posizioni.length && i<10;i++){
            System.out.println("posizione "+i+"tempo: "+tempi[i]+ "msec");
        }

       System.out.println("\n --- Classifiche sui tempi - ultimi 10 ---");
        
        for (int i=currEl-1; i>=0&&i>=currEl-10; i--){
            System.out.println("Posizione "+posizioni[i]+" tempo: "+tempi[i] + "msec");
        }
        
    }

}

 