package engine_lsd;

import java.util.*;

/**
 * Class Engine
 * 
 */
public class Engine {
	private PriorityQueue q;

	/**
	 * Number of expanded nodes
	 */
	public static int expandedNodes = 0;

	/**
	 * flag to activate verbose behaviour
	 */
	public static boolean verbose = false;

	/**
	 * limit value for queue insertion
	 */
	public static int limitValue = 1000;

	/**
	 * limit size for queue
	 */
	public static int limitSize = 100000;

	/**
	 * Constructor
	 */
	public Engine(State seed) {
		expandedNodes = 0;
		this.q = new PriorityQueue();
		this.q.put(seed);
	}

	/**
	 * Expand a state putting its successors in the queue
	 * 
	 * @return the solution state or null
	 */
	public State expand() {
		expandedNodes++;
		// if(expandedNodes%1000==1)System.out.println(" "+this);
		if (q.isEmpty())
			return null;
		if (verbose)
			System.out.println(" EXPAND " + this);
		State item = q.get(), son = null;
		if (verbose)
			System.out.println(item);

		for (Iterator e = item.successors(); e.hasNext();) {
			son = (State) e.next();
			if (son.value() == 0)
				return son;
			if (son.value() < limitValue)
				q.put(son);
		}
		return null;
	}

	/**
	 * Perform a complete search until goal is reached or resources exausted
	 * 
	 * @return the final state or null
	 */
	public State completeSearch() {
		State son = expand();
		while (!q.isEmpty() && q.size() < limitSize && son == null)
			son = expand();
		return son;
	}

	/**
	 * Return information on the search
	 * 
	 * @return a string with the queue size the minimal value and the number of
	 *         expanded nodes.
	 */
	public String toString() {
		return "Queue size = " + q.size() + ", min value " + q.look().value()
				+ ", expanded nodes " + expandedNodes;
	}
}