package sudoku_human;
/*
 Created on 4-set-2005
 
 one line to give the program's name and an idea of what it does.
 Copyright (C) 2005  Marco Cornolti (cornolti@cli.di.unipi.it)
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * Classe che definisce le proprieta' di una casella del Sudoku
 * (ovvero le possibilita' gia' scartate e quelle ancora da scartare).
 * 
 * @author Marco Cornolti
 * 
 */
public class Casella {
	private boolean[] numeriPossibili = new boolean[9];
	
	/**Costruisce una casella a partire dalle possibilita' non ancora scartate.
	 * @param possibilita array contenente gli interi [1-9] delle possibilita'
	 * non ancora scartate
	 */
	public Casella (int[] possibilita) {
	    for (int i=0;i<possibilita.length;i++) this.numeriPossibili[possibilita[i]-1]=true;
	}
	
	/**Costruisce una casella gia' definita.
	 * @param n il valore della casella definito.
	 */
	public Casella (int n)
	{
		if (n==0) for (int i=0;i<9;i++) numeriPossibili[i]=true;
		else {
			for (int i=0;i<9;i++) numeriPossibili[i]=false;
			numeriPossibili[n-1]=true;
		}
		
	}
	
	/**toglie una possibilita' da una casella
	 * @param n possibilita' da togliere
	 * @return true ses la casella e' stata modificata nelle possibilita'
	 */
	public boolean togliPossibilita (int n){
		if (n>0 && numeriPossibili[n-1]){
			numeriPossibili[n-1]=false;
			return true;
		}
		else return false;
	}
	/**verifica dell'esistenza di una possibilita' in una casella
	 * @param n numero della possibilita' [1-9]
	 * @return true ses esiste la possibilita' che la casella valga il numero passato
	 * per argomento
	 */
	public boolean getPossibilita (int n){
		return numeriPossibili[n-1];
	}
	/**il valore della casella
	 * @return 0 se non e' definita, altrimenti il valore della casella
	 */
	public int getValue(){
        
        int valore=0;
        boolean indefinito=false;
        for (int i=0;i<9;i++){
            if (indefinito && numeriPossibili[i]) return 0;
            indefinito = indefinito||numeriPossibili[i];
            if (numeriPossibili[i]) valore=i+1;
        }
        return valore;
    }
    /**
     * @return false ses esiste esattamente una possibilita' nella casella.
     */
    public boolean isUndefined(){
        
        boolean definito=false;
        for (int i=0;i<9;i++){
            if (definito && numeriPossibili[i]) return true;
            definito = definito||numeriPossibili[i];
        }
        return !definito;
    }
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		String result="";
		for (int i=0;i<9;i++) if (numeriPossibili[i]) result+=(i+1);
		return result;
		
	}

	/**
	 * @return un array di interi [1-9] con le possibilita'
	 */
	public int[] getPossibleValues() {
		int numPoss=0;
		for (int i=0;i<9;i++) if (numeriPossibili[i]) numPoss++;
		
		int[] possibiliValori=new int[numPoss];
		
		int iBool=0;
		int inseriti=0;
		while (inseriti<numPoss){
			if (numeriPossibili[iBool])possibiliValori[inseriti++]=iBool+1;
			iBool++;
		}
		return possibiliValori;
	}
	
	/**
	 * @return il numero di possibilita' di una casella
	 */
	public int getPossibleValuesCount() {
		int numPoss=0;
		for (int i=0;i<9;i++) if (numeriPossibili[i]) numPoss++;		
		return numPoss;
		}

	/**
     * definisce il valore di una casella
	 * @param n valore della casella [1-9]
	 */
	public void afferma(int n) {
		for (int i=0;i<9;i++) numeriPossibili[i]=false;
		numeriPossibili[n-1]=true;
	
	}
}
