package sudoku_lsd;

import java.util.*;

public class Sudoku {
	public static int LEVEL = 3;

	int[][] A = new int[9][9];

	boolean[][] INITIAL = new boolean[9][9];

	SudokuSet[][] P = new SudokuSet[9][9];

	boolean DEAD, SIMPLE;

	String moves;

	int nzeros, nmoves, goodmove = 0, smin, imin, jmin;

	Sudoku father;

	public Sudoku(int[][] tab) {
		this.A = tab;
		this.father = null;
		this.nmoves = 0;
		for (int i = 0; i < 9; i++)
			for (int j = 0; j < 9; j++)
				this.INITIAL[i][j] = tab[i][j] > 0;
		init();
		moves = "[";
	}

	public Sudoku(Sudoku S, int move) {
		if (S.DEAD)
			throw new IllegalArgumentException(" no possible move");

		int i0 = (move / 100);
		int j0 = (move / 10) % 10;
		int v = move % 10;
		if (S.A[i0][j0] != 0)
			throw new IllegalArgumentException(" illegal move");
		for (int i = 0; i < 9; i++)
			A[i] = (int[]) S.A[i].clone();
		A[i0][j0] = v;

		this.father = S;
		this.nmoves = S.nmoves + 1;
		this.moves = S.moves + " " + (110 + move);
		if (nmoves % 20 == 0)
			this.moves += "\n ";
		this.INITIAL = S.INITIAL;
		init();
	}

	public boolean isDead() {
		return DEAD;
	}

	public boolean isSimple() {
		return SIMPLE;
	}

	public boolean isInitial(int i, int j) {
		return INITIAL[i][j];
	}

	public Sudoku getFather() {
		return father;
	}

	public int zeros() {
		return nzeros;
	}

	public int choices() {
		return smin;
	}

	public int movesDone() {
		return nmoves;
	}

	public String movesList() {
		return moves + " ]";
	}

	public int getTable(int i, int j) {
		return A[i][j];
	}

	public SudokuSet getPossibilities(int i, int j) {
		return P[i][j];
	}

	public int getGoodMove() {
		if (SIMPLE)
			return goodmove;
		throw new IllegalArgumentException("  no simple table");
	}

	public Vector getMoves() {
		if (isDead())
			throw new IllegalArgumentException("  no moves ");
		Vector ls = new Vector();
		for (int k = 1; k < 10; k++)
			if (P[imin][jmin].contains(k))
				ls.addElement(new Integer(imin * 100 + jmin * 10 + k));
		return ls;
	}

	public double value() {
		if (DEAD)
			return 1000;
		return smin + nzeros / 100.;
	}

	// -----------------------------------------------------------------------
	private void init() {
		this.DEAD = false;
		this.SIMPLE = false;
		nzeros = 0;
		smin = 1000;
		for (int i = 0; i < 9; i++)
			for (int j = 0; j < 9; j++)
				if (A[i][j] == 0) {
					nzeros++;
					P[i][j] = possibilities(i, j);
					test1(i, j);
				}
		if (LEVEL % 2 == 1 && !SIMPLE && !DEAD)
			inverseSimplify();
		if (LEVEL > 1 && !SIMPLE && !DEAD) {
			fullSimplify();
			for (int i = 0; i < 9; i++)
				for (int j = 0; j < 9; j++)
					if (A[i][j] == 0)
						test1(i, j);
		}
		if (DEAD || nzeros == 0) {
			SIMPLE = false;
			smin = 0;
		}
	}

	private void test1(int i, int j) {
		int s = P[i][j].size();
		if (s < smin) {
			smin = s;
			imin = i;
			jmin = j;
		}
		if (s == 0)
			DEAD = true;
		if (s == 1) {
			SIMPLE = true;
			goodmove = i * 100 + j * 10 + P[i][j].first();
		}
	}

	private void inverseSimplify() {
		int nk, ik = 0, jk = 0;

		for (int i = 0; i < 9; i++)
			for (int k = 0; k <= 9; k++) {
				nk = 0;
				for (int j = 0; j < 9; j++)
					if (A[i][j] == k) {
						nk = 100;
						break;
					} else if (A[i][j] == 0 && P[i][j].contains(k)) {
						if (nk == 0) {
							nk++;
							ik = i;
							jk = j;
						} else {
							nk = 100;
							break;
						}
					}
				if (nk == 1) {
					SudokuSet set = new SudokuSet();
					set.add(k);
					P[ik][jk] = set;
					test1(ik, jk);
				}
			}
		if (SIMPLE)
			return;

		for (int j = 0; j < 9; j++)
			for (int k = 0; k <= 9; k++) {
				nk = 0;
				for (int i = 0; i < 9; i++)
					if (A[i][j] == k) {
						nk = 100;
						break;
					} else if (A[i][j] == 0 && P[i][j].contains(k)) {
						if (nk == 0) {
							nk++;
							ik = i;
							jk = j;
						} else {
							nk = 100;
							break;
						}
					}
				if (nk == 1) {
					SudokuSet set = new SudokuSet();
					set.add(k);
					P[ik][jk] = set;
					test1(ik, jk);
				}
			}
		if (SIMPLE)
			return;

		for (int i0 = 0; i0 < 9; i0 += 3)
			for (int j0 = 0; j0 < 9; j0 += 3)
				for (int k = 0; k <= 9; k++) {
					nk = 0;
					for (int i1 = 0; i1 < 3; i1++)
						for (int j1 = 0; j1 < 3; j1++) {
							int i = i0 + i1, j = j0 + j1;
							if (A[i][j] == k) {
								nk = 100;
								break;
							} else if (A[i][j] == 0 && P[i][j].contains(k)) {
								if (nk == 0) {
									nk++;
									ik = i;
									jk = j;
								} else {
									nk = 100;
									break;
								}
							}
						}
					if (nk == 1) {
						SudokuSet set = new SudokuSet();
						set.add(k);
						P[ik][jk] = set;
						test1(ik, jk);
					}
				}
	}

	private SudokuSet possibilities(int i0, int j0) {
		SudokuSet set = new SudokuSet();
		for (int i = 0; i < 9; i++)
			if (i != i0 && A[i][j0] > 0)
				set.add(A[i][j0]);
		for (int j = 0; j < 9; j++)
			if (j != j0 && A[i0][j] > 0)
				set.add(A[i0][j]);
		int ib = 3 * (i0 / 3), jb = 3 * (j0 / 3);
		for (int i = 0; i < 3; i++)
			for (int j = 0; j < 3; j++)
				if (i + ib != i0 && j + jb != j0 && A[i + ib][j + jb] > 0)
					set.add(A[i + ib][j + jb]);
		return set.complement();
	}

	private void fullSimplify() {
		SudokuSimplify SS = new SudokuSimplify(A);
		for (int i = 0; i < 9; i++) {
			for (int j = 0; j < 9; j++)
				if (A[i][j] == 0)
					SS.push(i, j, P[i][j]);
			SS.simplify(P);
		}
		for (int j = 0; j < 9; j++) {
			for (int i = 0; i < 9; i++)
				if (A[i][j] == 0)
					SS.push(i, j, P[i][j]);
			SS.simplify(P);
		}
		for (int i0 = 0; i0 < 9; i0 += 3)
			for (int j0 = 0; j0 < 9; j0 += 3) {
				for (int i = 0; i < 3; i++)
					for (int j = 0; j < 3; j++)
						if (A[i + i0][j + j0] == 0)
							SS.push(i + i0, j + j0, P[i + i0][j + j0]);
				SS.simplify(P);
			}
	}
}

class SudokuSimplify {
	int[][] A;

	int[] rows = new int[9];

	int[] cols = new int[9];

	SudokuSet[] ppp = new SudokuSet[9];

	int np = 0;

	public SudokuSimplify(int[][] A) {
		this.A = A;
		this.np = 0;
	}

	public void push(int i, int j, SudokuSet p) {
		rows[np] = i;
		cols[np] = j;
		ppp[np] = p;
		++np;
	}

	public void simplify(SudokuSet[][] P) {
		if (np <= 2) {
			np = 0;
			return;
		}
		if (np > 2)
			if (s2())
				for (int i = 0; i < np; i++)
					P[rows[i]][cols[i]] = ppp[i];
		np = 0;
	}

	private boolean s2() {
		boolean done = false;
		for (int i1 = 0; i1 < np; i1++)
			if (ppp[i1].size() == 2)
				for (int i2 = 0; i2 < np; i2++)
					if (i1 != i2 && ppp[i2].size() == 2) {
						SudokuSet u = SudokuSet.union(ppp[i1], ppp[i2]);
						if (u.size() == 2)
							for (int i = 0; i < np; i++)
								if (i != i1 && i != i2) {
									done = true;
									SudokuSet r = SudokuSet.difference(ppp[i],
											u);
									ppp[i] = r;
								}
					}
		return done;
	}
}